@@ -1,7 +1,7 @@
 /*
  *  xfdesktop - xfce4's desktop manager
  *
- *  Copyright (c) 2006 Brian Tarricone, <bjt23@cornell.edu>
+ *  Copyright (c) 2006-2007 Brian Tarricone, <bjt23@cornell.edu>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include <gobject/gmarshal.h>
 
 #include "xfdesktop-icon.h"
+#include "xfdesktop-marshal.h"
 
 struct _XfdesktopIconPrivate
 {
@@ -44,12 +45,9 @@ enum {
     SIG_POS_CHANGED,
     SIG_SELECTED,
     SIG_ACTIVATED,
-    SIG_MENU_POPUP,
     SIG_N_SIGNALS,
 };
 
-static void xfdesktop_icon_class_init(XfdesktopIconClass *klass);
-static void xfdesktop_icon_init(XfdesktopIcon *icon);
 
 static guint __signals[SIG_N_SIGNALS] = { 0, };
 
@@ -57,38 +55,6 @@ static guint __signals[SIG_N_SIGNALS] = { 0, };
 G_DEFINE_ABSTRACT_TYPE(XfdesktopIcon, xfdesktop_icon, G_TYPE_OBJECT)
 
 
-static void
-xfdesktop_icon_marshal_BOOLEAN__VOID(GClosure *closure,
-                                     GValue *return_value,
-                                     guint n_param_values,
-                                     const GValue *param_values,
-                                     gpointer invocation_hint,
-                                     gpointer marshal_data)
-{
-    typedef gboolean (*GMarshalFunc_BOOLEAN__VOID)(gpointer, gpointer);
-    register GMarshalFunc_BOOLEAN__VOID callback;
-    register GCClosure *cc = (GCClosure *)closure;
-    register gpointer data1, data2;
-    gboolean ret = FALSE;
-    
-    g_return_if_fail(return_value);
-    g_return_if_fail(n_param_values == 1);
-    
-    if(G_CCLOSURE_SWAP_DATA(closure)) {
-        data1 = closure->data;
-        data2 = g_value_peek_pointer(param_values + 0);
-    } else {
-        data1 = g_value_peek_pointer(param_values + 0);
-        data2 = closure->data;
-    }
-
-    callback = (GMarshalFunc_BOOLEAN__VOID)(marshal_data ? marshal_data : cc->callback);
-
-    ret = callback(data1, data2);
-    
-    g_value_set_boolean(return_value, ret);
-}
-
 static void
 xfdesktop_icon_class_init(XfdesktopIconClass *klass)
 {
@@ -137,17 +103,8 @@ xfdesktop_icon_class_init(XfdesktopIconClass *klass)
                                                             activated),
                                             g_signal_accumulator_true_handled,
                                             NULL,
-                                            xfdesktop_icon_marshal_BOOLEAN__VOID,
+                                            xfdesktop_marshal_BOOLEAN__VOID,
                                             G_TYPE_BOOLEAN, 0);
-    
-    __signals[SIG_MENU_POPUP] = g_signal_new("menu-popup",
-                                             XFDESKTOP_TYPE_ICON,
-                                             G_SIGNAL_RUN_LAST,
-                                             G_STRUCT_OFFSET(XfdesktopIconClass,
-                                                             menu_popup),
-                                             NULL, NULL,
-                                             g_cclosure_marshal_VOID__VOID,
-                                             G_TYPE_NONE, 0);
 }
 
 static void
@@ -300,19 +257,20 @@ xfdesktop_icon_peek_tooltip(XfdesktopIcon *icon)
 }
 
 /*< optional >*/
-GtkWidget *
-xfdesktop_icon_get_popup_menu(XfdesktopIcon *icon)
+gboolean
+xfdesktop_icon_populate_context_menu(XfdesktopIcon *icon,
+                                     GtkWidget *menu)
 {
     XfdesktopIconClass *klass;
     
-    g_return_val_if_fail(XFDESKTOP_IS_ICON(icon), NULL);
+    g_return_val_if_fail(XFDESKTOP_IS_ICON(icon), FALSE);
     
     klass = XFDESKTOP_ICON_GET_CLASS(icon);
     
-    if(!klass->get_popup_menu)
-        return NULL;
+    if(!klass->populate_context_menu)
+        return FALSE;
     
-    return klass->get_popup_menu(icon);
+    return klass->populate_context_menu(icon, menu);
 }
 
 GtkWidget *
@@ -364,10 +322,3 @@ xfdesktop_icon_activated(XfdesktopIcon *icon)
     
     return ret;
 }
-
-void
-xfdesktop_icon_menu_popup(XfdesktopIcon *icon)
-{
-    g_return_if_fail(XFDESKTOP_IS_ICON(icon));
-    g_signal_emit(G_OBJECT(icon), __signals[SIG_MENU_POPUP], 0, NULL);
-}
